package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            persistEntities();
            executeQuery();
            executeQuery2();
        } finally {
            entityManagerFactory.close();
        }
    }

    public static void persistEntities() {
        System.out.println("-- persisting entities --");
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        getProductInventoryList().forEach(em::persist);
        getOrderItemList().forEach(em::persist);
        em.getTransaction().commit();

        System.out.println("-- entities persisted --");
        em.createQuery("SELECT p FROM ProductInventory p")
          .getResultList()
          .forEach(System.out::println);

        em.createQuery("SELECT o FROM OrderItem o")
          .getResultList()
          .forEach(System.out::println);
        em.close();

    }

    private static void executeQuery() {
        System.out.println("-- finding OrderItem which have qty > ALL inventories qty --");
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery("SELECT o FROM OrderItem o"
                + " WHERE o.quantity > ALL (SELECT p.quantity from ProductInventory p"
                + " WHERE p.productName= o.productName)");
        List<OrderItem> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void executeQuery2() {
        System.out.println("-- finding OrderItem which have qty > ANY inventories qty --");
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery("SELECT o FROM OrderItem o"
                + " WHERE o.quantity > ANY (SELECT p.quantity from ProductInventory p"
                + " WHERE p.productName= o.productName)");
        List<OrderItem> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static List<ProductInventory> getProductInventoryList() {
        List<ProductInventory> list = new ArrayList<>();
        list.add(ProductInventory.of("Monitor", 10, "Vritville"));
        list.add(ProductInventory.of("Monitor", 50, "Lonpore"));
        list.add(ProductInventory.of("Desktop Computer", 15, "Vritville"));
        list.add(ProductInventory.of("Desktop Computer", 20, "Lonpore"));
        list.add(ProductInventory.of("Laptop", 25, "Vritville"));
        list.add(ProductInventory.of("Laptop", 30, "Lonpore"));
        list.add(ProductInventory.of("SSD", 45, "Vritville"));
        list.add(ProductInventory.of("SSD", 30, "Lonpore"));
        return list;
    }

    private static List<OrderItem> getOrderItemList() {
        List<OrderItem> list = new ArrayList<>();
        list.add(OrderItem.of("Monitor", 60));
        list.add(OrderItem.of("Desktop Computer", 9));
        list.add(OrderItem.of("Laptop", 25));
        list.add(OrderItem.of("SSD", 40));
        return list;
    }
}